@@ -19,7 +19,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-/* $Id: klflatexsymbols.cpp 603 2011-02-26 23:14:55Z phfaist $ */
+/* $Id: klflatexsymbols.cpp 997 2017-01-15 09:55:24Z phfaist $ */
 
 #include <stdio.h>
 
@@ -27,12 +27,13 @@
 #include <QDir>
 #include <QTextStream>
 #include <QScrollArea>
+#include <QRegExp>
 #include <QList>
 #include <QStringList>
 #include <QProgressDialog>
 #include <QGridLayout>
 #include <QPushButton>
-#include <QMap>
+#include <QHash>
 #include <QStackedWidget>
 #include <QLineEdit>
 #include <QMessageBox>
@@ -41,7 +42,7 @@
 #include <QCloseEvent>
 #include <QDebug>
 #include <QPainter>
-#include <QPlastiqueStyle>
+#include <QStyleFactory>
 
 #include <QDomDocument>
 #include <QDomElement>
@@ -51,14 +52,29 @@
 #include <ui_klflatexsymbols.h>
 
 #include <klfpixmapbutton.h>
+#include <klfrelativefont.h>
+
 #include "klfmain.h"
 #include "klfconfig.h"
 #include "klflatexsymbols.h"
 
+#include "klflatexsymbols_p.h"
+
 
 
 // ------------------
 
+KLF_EXPORT QDebug operator<<(QDebug str, const KLFLatexSymbol& s)
+{
+  return str << "KLFLatexSymbol["<<s.symbol
+	     <<(s.symbol_option?"[]":"")
+	     <<(s.symbol_numargs>0?qPrintable("{"+QString::number(s.symbol_numargs)+"x}"):"")
+	     <<";preview="<<(s.previewlatex.size()?qPrintable("`"+s.previewlatex+"'"):"")
+	     <<";preamblelist/size="<<s.preamble.size()
+	     <<";txtmd="<<s.textmode<<";hidden="<<s.hidden
+	     <<";kw="<<s.keywords;
+}
+
 
 KLFLatexSymbol::KLFLatexSymbol(const QDomElement& e)
   : symbol(), preamble(), textmode(false), bbexpand(), hidden(false)
@@ -69,9 +85,9 @@ KLFLatexSymbol::KLFLatexSymbol(const QDomElement& e)
   for (k = 0; k < usepackagelist.size(); ++k) {
     QString package = usepackagelist.at(k).toElement().attribute("name");
     if (package[0] == '[' || package[0] == '{')
-      preamble.append(QString::fromAscii("\\usepackage%1").arg(package));
+      preamble.append(QString("\\usepackage%1").arg(package));
     else
-      preamble.append(QString::fromAscii("\\usepackage{%1}").arg(package));
+      preamble.append(QString("\\usepackage{%1}").arg(package));
   }
   QDomNodeList preamblelinelist = e.elementsByTagName("preambleline");
   for (k = 0; k < preamblelinelist.size(); ++k) {
@@ -101,16 +117,89 @@ KLFLatexSymbol::KLFLatexSymbol(const QDomElement& e)
   if (latexlist.size() != 1) {
     fprintf(stderr, "WARNING: Expected single <latex>...</latex> in symbol entry!\n");
   }
-  if (latexlist.size() == 0)
+  if (latexlist.size() == 0) {
     return;
-  symbol = latexlist.at(0).toElement().text();
+  }
+  QRegExp rx_bool("\\s*(y(?:es)?|t(?:rue)?|on|1)|(n(?:o)?|f(?:alse)?|off|0)\\s*", Qt::CaseInsensitive);
+
+  QDomElement latexnode = latexlist.at(0).toElement();
+  symbol = latexnode.text();
+  QString symopt = latexnode.attribute("option", "no");
+  bool boolok = rx_bool.exactMatch(symopt);
+  if (!boolok) {
+    qWarning()<<KLF_FUNC_NAME<<": Bad boolean format "<<symopt;
+    symbol_option = false;
+  } else {
+    // if cap(1) is non-empty, TRUE; if cap(2) is non-empty then FALSE
+    symbol_option = rx_bool.cap(1).size();
+  }
+  QString nargs = latexnode.attribute("numargs", "0");
+  bool uintok = true;
+  symbol_numargs = nargs.toUInt(&uintok);
+  if (!uintok) {
+    qWarning()<<KLF_FUNC_NAME<<": Bad integer format "<<nargs;
+  }
+
+  // latex preview code
+  QDomNodeList previewlatexlist = e.elementsByTagName("preview-latex");
+  if (previewlatexlist.size() > 1) {
+    fprintf(stderr, "WARNING: Expected at most one <preview-latex>...</preview-latex> in symbol entry!\n");
+  }
+  if (previewlatexlist.size() > 0) {
+    QDomElement previewlatexnode = previewlatexlist.at(0).toElement();
+    previewlatex = previewlatexnode.text();
+  } else {
+    previewlatex = QString();
+  }
+
+  // keywords
+  QDomNodeList keywordslist = e.elementsByTagName("keywords");
+  for (k = 0; k < keywordslist.size(); ++k) {
+    QStringList kw;
+    kw = keywordslist.at(k).toElement().text().split(QRegExp("\\s+"), QString::SkipEmptyParts);
+    keywords << kw;
+  }
 
   klfDbg("read symbol "<<symbol<<" hidden="<<hidden);
 }
 
+QString KLFLatexSymbol::symbolWithArgs() const
+{
+  QString s = symbol;
+  if (symbol_option)
+    s += "[]";
+  int n = symbol_numargs;
+  while (n--)
+    s += "{}";
+  return s;
+}
+
+QString KLFLatexSymbol::latexCodeForPreview() const
+{
+  if (previewlatex.size())
+    return previewlatex;
+
+  static const QString dummyopt = "n";
+  static const QStringList dummyargs =
+    QStringList()<<"x"<<"y"<<"z"<<"w"<<"a"<<"b"<<"c"<<"d"<<"e"<<"f";
+  int k;
+
+  QString s = symbol;
+  if (symbol_option)
+    s += "["+dummyopt+"]";
+  for (k = 0; k < symbol_numargs; ++k)
+    s += "{"+dummyargs[k%dummyargs.size()]+"}";
+
+  return s;
+}
+
+
 KLF_EXPORT bool operator==(const KLFLatexSymbol& a, const KLFLatexSymbol& b)
 {
   return a.symbol == b.symbol &&
+    a.symbol_numargs == b.symbol_numargs &&
+    a.symbol_option == b.symbol_option &&
+    a.previewlatex == b.previewlatex &&
     a.textmode == b.textmode &&
     a.preamble == b.preamble &&
     a.bbexpand.t == b.bbexpand.t &&
@@ -120,35 +209,35 @@ KLF_EXPORT bool operator==(const KLFLatexSymbol& a, const KLFLatexSymbol& b)
     a.hidden == b.hidden;
 }
 
-KLF_EXPORT bool operator<(const KLFLatexSymbol& a, const KLFLatexSymbol& b)
-{
-  if (a.symbol != b.symbol)
-    return a.symbol < b.symbol;
-  if (a.textmode != b.textmode)
-    return a.textmode < b.textmode;
-  if (a.preamble.size() != b.preamble.size())
-    return a.preamble.size() < b.preamble.size();
-  int k;
-  for (k = 0; k < a.preamble.size(); ++k)
-    if (a.preamble[k] != b.preamble[k])
-      return a.preamble[k] < b.preamble[k];
-  // a and b seem to be equal
-  return false;
-}
-
 QDataStream& operator<<(QDataStream& stream, const KLFLatexSymbol& s)
 {
-  return stream << s.symbol << s.preamble << (quint8)s.textmode
-		<< (qint16)s.bbexpand.t << (qint16)s.bbexpand.r
-		<< (qint16)s.bbexpand.b << (qint16)s.bbexpand.l << (quint8)s.hidden;
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+  return stream << s.symbol
+		<< (quint8)s.symbol_option
+		<< (quint8)s.symbol_numargs
+		<< s.previewlatex
+		<< s.preamble
+		<< (quint8)s.textmode
+		<< (qint16)s.bbexpand.t << (qint16)s.bbexpand.r << (qint16)s.bbexpand.b << (qint16)s.bbexpand.l
+		<< (quint8)s.hidden
+		<< s.keywords;
 }
 
 QDataStream& operator>>(QDataStream& stream, KLFLatexSymbol& s)
 {
-  quint8 textmode, hidden;
+  quint8 symopt, symnargs, textmode, hidden;
   struct { qint16 t, r, b, l; } readbbexpand;
-  stream >> s.symbol >> s.preamble >> textmode >> readbbexpand.t >> readbbexpand.r
-	 >> readbbexpand.b >> readbbexpand.l >> hidden;
+  stream >> s.symbol
+	 >> symopt
+	 >> symnargs
+	 >> s.previewlatex
+	 >> s.preamble
+	 >> textmode
+	 >> readbbexpand.t >> readbbexpand.r >> readbbexpand.b >> readbbexpand.l
+	 >> hidden
+	 >> s.keywords;
+  s.symbol_option = symopt;
+  s.symbol_numargs = symnargs;
   s.bbexpand.t = readbbexpand.t;
   s.bbexpand.r = readbbexpand.r;
   s.bbexpand.b = readbbexpand.b;
@@ -163,45 +252,21 @@ QDataStream& operator>>(QDataStream& stream, KLFLatexSymbol& s)
 // -----------------------------------------------------------
 
 
-KLFLatexSymbolsCache * KLFLatexSymbolsCache::staticCache = NULL;
-
-static QString __rel_cache_file = QString();
-static QString relcachefile()
-{
-  if (__rel_cache_file.isEmpty())
-    __rel_cache_file =
-      QString("/symbolspixmapcache-klf%1").arg(KLF_DATA_STREAM_APP_VERSION);
-  return __rel_cache_file;
-}
-
-KLFLatexSymbolsCache::KLFLatexSymbolsCache()
+KLFLatexSymbolsCache::KLFLatexSymbolsCache(QObject * parent)
+  : QObject(parent)
 {
   KLF_DEBUG_TIME_BLOCK(KLF_FUNC_NAME) ;
-  // load the cache
 
-  QStringList cachefiles;
-  cachefiles
-    << klfconfig.homeConfigDir+relcachefile()
-    << ":/data/symbolspixmapcache_base" ;
-  int k;
-  bool ok = false;
-  for (k = 0; !ok && k < cachefiles.size(); ++k) {
-    // ??: do the two attempts here apply a datastream version to the header only, or to
-    //     the data too?
-    klfDbg("trying to load from "<<cachefiles[k]) ;
-    ok = (  loadCacheFrom(cachefiles[k], QDataStream::Qt_4_4)
-	    == KLFLatexSymbolsCache::Ok  ) ;
-    if (!ok) {
-      klfDbg("trying to load from "<<cachefiles[k]<<" with default header datastream version") ;
-      ok = (  loadCacheFrom(cachefiles[k], -1)
-	      == KLFLatexSymbolsCache::Ok  ) ;
-    }
-  }
-  if ( ! ok ) {
-    qWarning() << KLF_FUNC_NAME << ": error finding and reading cache file!";
-  }
+  KLF_INIT_PRIVATE(KLFLatexSymbolsCache) ;
 
-  flag_modified = false;
+  d->device_pixel_ratio = 1.0;
+
+  d->flag_modified = false;
+}
+
+KLFLatexSymbolsCache::~KLFLatexSymbolsCache()
+{
+  KLF_DELETE_PRIVATE ;
 }
 
 int KLFLatexSymbolsCache::loadCacheStream(QDataStream& stream)
@@ -221,9 +286,9 @@ int KLFLatexSymbolsCache::loadCacheStream(QDataStream& stream)
 
   // stream is now ready to read
 
-  stream >> cache;
+  stream >> d->cache;
 
-  flag_modified = false;
+  d->flag_modified = false;
   return 0;
 }
 
@@ -231,151 +296,119 @@ int KLFLatexSymbolsCache::saveCacheStream(QDataStream& stream)
 {
   klfDataStreamWriteHeader(stream, "KLATEXFORMULA_SYMBOLS_PIXMAP_CACHE");
   // stream is now ready to be written
-  stream << cache;
-  flag_modified = false;
+  stream << d->cache;
+  d->flag_modified = false;
   return 0;
 }
 
-QPixmap KLFLatexSymbolsCache::getPixmap(const KLFLatexSymbol& sym, bool fromcacheonly)
+bool KLFLatexSymbolsCache::cacheNeedsSave() const
 {
-  klfDbg("sym.symbol="<<sym.symbol<<" fromCacheOnly="<<fromcacheonly) ;
+  return d->flag_modified;
+}
 
-  if (cache.contains(sym)) {
-    klfDbg("Found symbol in cache! pixmap is null="<<cache[sym].isNull()<<"; sym.preamble="<<sym.preamble.join(";"));
-    return cache[sym];
+QPixmap KLFLatexSymbolsCache::getSymbolPixmap(const KLFLatexSymbol& sym)
+{
+  klfDbg("sym.symbol="<<sym.symbol) ;
+  klfDbg("full symbol: "<<sym) ;
+
+  if (d->cache.contains(sym)) {
+    klfDbg("Found symbol in cache! pixmap is null="<<d->cache[sym].pix.isNull()
+	   <<"; sym.preamble="<<sym.preamble.join(";"));
+    return QPixmap(d->cache[sym].pix);
   }
 
-  if (fromcacheonly) {
-    // if we weren't able to load it from cache, show failed icon
-    return QPixmap(":/pics/badsym.png");
-  }
+  // if we weren't able to load it from cache, show failed icon
+  return QPixmap(":/pics/badsym.png");
+}
+
+void KLFLatexSymbolsCache::setSymbolPixmap(const KLFLatexSymbol & sym, const QPixmap & pix)
+{
+  klfDbg("sym.symbol="<<sym.symbol) ;
+  klfDbg("full symbol: "<<sym) ;
 
   {
     KLF_DEBUG_TIME_BLOCK(KLF_FUNC_NAME+"/clean cache duplicate test") ;
     // clean cache: make sure there are no two duplicate symbols (this is for the popup hint parser,
     // so that it doesn't detect old symbols in the cache)
-    // This is done only if fromcache is false, so as to perform the check only on first pass
-    // when generating the symbol cache.
-    QMap<KLFLatexSymbol,QPixmap>::iterator it = cache.begin();
-    while (it != cache.end()) {
+    QHash<KLFLatexSymbol,SymbolInfo>::iterator it = d->cache.begin();
+    while (it != d->cache.end()) {
       klfDbg("Testing symbol "<<it.key().symbol<<",preamble="<<it.key().preamble.join(",")
 	     << "for being a duplicate of "<<sym.symbol);
-      if (it.key().symbol == sym.symbol) {
-	klfDbg("erasing duplicate.");
-	it = cache.erase(it); // erase old symbol entry
+      if (it.key() == sym && !it.value().confirmed) {
+	klfDbg("erasing duplicate marked unconfirmed.");
+	it = d->cache.erase(it); // erase old symbol entry
       } else {
 	++it;
       }
     }
   }
 
-  if (sym.hidden) {
-    // special treatment for hidden symbols
-    // insert a QPixmap() into cache and return it
-    klfDbg("symbol is hidden. Assigning NULL pixmap.") ;
-    cache[sym] = QPixmap();
-    return QPixmap();
-  }
+  d->cache[sym].pix = pix;
+  d->cache[sym].confirmed = true;
 
-  const float mag = 4.0;
-
-  KLFBackend::klfInput in;
-  in.latex = sym.symbol;
-  in.mathmode = sym.textmode ? "..." : "\\[ ... \\]";
-  in.preamble = sym.preamble.join("\n")+"\n";
-  in.fg_color = qRgb(0,0,0);
-  in.bg_color = qRgba(255,255,255,0); // transparent Bg
-  in.dpi = (int)(mag * 150);
-
-  backendsettings.epstopdfexec = ""; // don't waste time making PDF, we don't need it
-  backendsettings.tborderoffset = sym.bbexpand.t;
-  backendsettings.rborderoffset = sym.bbexpand.r;
-  backendsettings.bborderoffset = sym.bbexpand.b;
-  backendsettings.lborderoffset = sym.bbexpand.l;
-
-  KLFBackend::klfOutput out = KLFBackend::getLatexFormula(in, backendsettings);
-
-  if (out.status != 0) {
-    qWarning()
-      <<KLF_FUNC_NAME
-      <<QString(":ERROR: Can't generate preview for symbol %1 : status %2 !\n\tError: %3\n")
-      .arg(sym.symbol).arg(out.status).arg(out.errorstr);
-    return QPixmap(":/pics/badsym.png");
-  }
-
-  flag_modified = true;
-
-  QImage scaled = out.result.scaled((int)(out.result.width() / mag),
-				    (int)(out.result.height() / mag),
-				    Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-  QPixmap pix = QPixmap::fromImage(scaled);
-  cache[sym] = pix;
-
-  klfDbg("Ran getLatexFormula(), got the pixmap. Returning.") ;
-
-  return pix;
+  d->flag_modified = true;
 }
 
-int KLFLatexSymbolsCache::precacheList(const QList<KLFLatexSymbol>& list, bool userfeedback,
-				       QWidget *parent)
+void KLFLatexSymbolsCache::setSymbolPixmaps(const QHash<KLFLatexSymbol,QPixmap>& sympixmaps)
 {
-  KLF_DEBUG_TIME_BLOCK(KLF_FUNC_NAME) ;
-
-  QProgressDialog *pdlg = NULL;
-
-  if (userfeedback) {
-    pdlg = new QProgressDialog(QObject::tr("Please wait while generating symbol previews ... "),
-			       QObject::tr("Skip"), 0, list.size()-1, parent);
-    /** TODO: we should do a first pass to see which symbols are missing, then
-     * on a second pass generate those symbols with a progress dialog... */
-    //  pdlg->setMinimumDuration(15000);
-    pdlg->setWindowModality(Qt::WindowModal);
-    pdlg->setModal(true);
-    pdlg->setValue(0);
+  for (QHash<KLFLatexSymbol,QPixmap>::const_iterator it = sympixmaps.begin();
+       it != sympixmaps.end();  ++it) {
+    KLFLatexSymbol sym = it.key();
+    QPixmap pix = it.value();
+    klfDbg("setting symbol pixmap for " << sym.symbol ) ;
+    setSymbolPixmap(sym, pix);
   }
-
-  for (int i = 0; i < list.size(); ++i) {
-    if (userfeedback) {
-      // get events for cancel button (for example)
-      qApp->processEvents();
-      if (pdlg->wasCanceled()) {
-	delete pdlg;
-	return 1;
-      }
-      pdlg->setValue(i);
-    }
-    getPixmap(list[i], false);
-  }
-
-  if (userfeedback) {
-    delete pdlg;
-  }
-
-  return 0;
-};
+}
 
 
 
 void KLFLatexSymbolsCache::setBackendSettings(const KLFBackend::klfSettings& settings)
 {
-  backendsettings = settings;
+  d->backendsettings = settings;
 }
 
+void KLFLatexSymbolsCache::setDevicePixelRatio(qreal ratio)
+{
+  d->device_pixel_ratio = ratio;
+}
+
+bool KLFLatexSymbolsCache::contains(const KLFLatexSymbol & sym)
+{
+  return d->cache.contains(sym);
+}
+
+
+
 KLFLatexSymbol KLFLatexSymbolsCache::findSymbol(const QString& symbolCode)
 {
-  for (QMap<KLFLatexSymbol,QPixmap>::const_iterator it = cache.begin();
-       it != cache.end(); ++it) {
-    if (it.key().symbol == symbolCode)
+  for (QHash<KLFLatexSymbol,SymbolInfo>::const_iterator it = d->cache.begin();
+       it != d->cache.end(); ++it) {
+    if (it.key().symbol == symbolCode) {
       return it.key();
+    }
   }
   return KLFLatexSymbol();
 }
 
+QList<KLFLatexSymbol> KLFLatexSymbolsCache::findSymbols(const QString& symbolCode)
+{
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+  QList<KLFLatexSymbol> slist;
+  for (QHash<KLFLatexSymbol,SymbolInfo>::const_iterator it = d->cache.begin();
+       it != d->cache.end(); ++it) {
+    if (it.key().symbol == symbolCode) {
+      klfDbg("found symbol: "<<it.key());
+      slist << it.key();
+    }
+  }
+  return slist;
+}
+
 QStringList KLFLatexSymbolsCache::symbolCodeList()
 {
   QStringList l;
-  for (QMap<KLFLatexSymbol,QPixmap>::const_iterator it = cache.begin();
-       it != cache.end(); ++it)
+  for (QHash<KLFLatexSymbol,SymbolInfo>::const_iterator it = d->cache.begin();
+       it != d->cache.end(); ++it)
     l << it.key().symbol;
   return l;
 }
@@ -385,14 +418,30 @@ QPixmap KLFLatexSymbolsCache::findSymbolPixmap(const QString& symbolCode)
   KLFLatexSymbol sym = findSymbol(symbolCode);
   if (sym.symbol.isEmpty()) {
     // invalid symbol
-    qWarning()<<KLF_FUNC_NAME<<": Can't find symbol "<<symbolCode<<".";
+    klfWarning("Can't find symbol "<<symbolCode);
     return QPixmap();
   }
   // return the pixmap from cache
-  return cache[sym];
+  return d->cache[sym].pix;
 }
 
-
+void KLFLatexSymbolsCache::debugDump()
+{
+#ifdef KLF_DEBUG
+  klfDbg("dumping cache.");
+  QHash<KLFLatexSymbol,SymbolInfo>::const_iterator it;
+  int k;
+  for (it = d->cache.begin(), k = 0; it != d->cache.end(); ++it, ++k) {
+    const KLFLatexSymbol& s = it.key();
+    const QPixmap& p = it.value().pix;
+    qDebug("  #%4d  %s (%s)  prembl/sz=%d  txtmd=%s  hid=%s  pix/sz=(%d,%d) confirmed=%s", k,
+	   qPrintable(s.symbolWithArgs()), qPrintable(s.previewlatex),
+	   s.preamble.size(), s.textmode?"yes":"no", s.hidden?"yes":"no",
+	   p.width(), p.height(), it.value().confirmed?"yes":"no"
+	   );
+  }
+#endif
+}
 
 
 
@@ -407,35 +456,60 @@ int KLFLatexSymbolsCache::loadCacheFrom(const QString& fname, int version)
     return -1;
   }
   QDataStream ds(&f);
-  if (version >= 0)
+  if (version >= 0) {
     ds.setVersion(version);
+  }
   int r = loadCacheStream(ds);
   return r;
 }
 
+void KLFLatexSymbolsCache::loadKlfCache()
+{
+  // load the klf cache
 
-// static
-KLFLatexSymbolsCache * KLFLatexSymbolsCache::theCache()
-{
-  if (staticCache == NULL) {
-    staticCache = new KLFLatexSymbolsCache;
+  QStringList cachefiles;
+  cachefiles
+    << klfconfig.homeConfigDir + QLatin1String("/") + d->klf_cache_file_name()
+    << ":/data/base_"+d->klf_cache_file_name()
+    ;
+
+  int k;
+  bool ok = false;
+  for (k = 0; !ok && k < cachefiles.size(); ++k) {
+    klfDbg("trying to load from "<<cachefiles[k]) ;
+    ok = (  loadCacheFrom(cachefiles[k], QDataStream::Qt_4_4)
+	    == KLFLatexSymbolsCache::Ok  ) ;
+    if (!ok) {
+      klfDbg("trying to load from "<<cachefiles[k]<<" with default header datastream version") ;
+      ok = (  loadCacheFrom(cachefiles[k], -1)
+	      == KLFLatexSymbolsCache::Ok  ) ;
+    }
   }
-  return staticCache;
+  if ( ! ok ) {
+    klfWarning("Can't find and read cache file!");
+  }
+
 }
-// static
-void KLFLatexSymbolsCache::saveTheCache()
+
+void KLFLatexSymbolsCache::saveToKlfCache()
 {
-  if (staticCache->cacheNeedsSave()) {
-    QString s = klfconfig.homeConfigDir + relcachefile();
-    QFile f(s);
+  saveCache(klfconfig.homeConfigDir + QLatin1String("/") + d->klf_cache_file_name());
+}
+void KLFLatexSymbolsCache::saveCache(const QString & fname)
+{
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+
+  if (cacheNeedsSave()) {
+    // QString s = klfconfig.homeConfigDir + relcachefile();
+    QFile f(fname);
     if ( ! f.open(QIODevice::WriteOnly) ) {
-      qWarning() << KLF_FUNC_NAME<< "Can't save cache to file "<< s << "!";
+      klfWarning("Can't save cache to file " << fname);
       return;
     }
     QDataStream ds(&f);
     ds.setVersion(QDataStream::Qt_4_4);
-    staticCache->saveCacheStream(ds);
-    klfDbg("Saved cache to file "<<s);
+    saveCacheStream(ds);
+    klfDbg("Saved cache to file "<<fname);
   }
 }
 
@@ -484,30 +558,37 @@ void KLFLatexSymbolsView::appendSymbolList(const QList<KLFLatexSymbol>& symbols)
       _symbols.append(symbols[k]);
 }
 
-void KLFLatexSymbolsView::buildDisplay()
+void KLFLatexSymbolsView::buildDisplay(KLFLatexSymbolsCache * cache)
 {
   KLF_DEBUG_TIME_BLOCK(KLF_FUNC_NAME) ;
-#ifdef Q_WS_MAC
-  QStyle *myStyle = new QPlastiqueStyle;
-  QPalette pal = palette();
-  pal.setColor(QPalette::Window, QColor(206,207,233));
-  pal.setColor(QPalette::Base, QColor(206,207,233));
-  pal.setColor(QPalette::Button, QColor(206,207,233));
+#ifdef KLF_WS_MAC
+  QStyle *buttonstyle = QStyleFactory::create("fusion");
+//  QPalette pal = palette();
+//  pal.setColor(QPalette::Window, QColor(206,207,233));
+//  pal.setColor(QPalette::Base, QColor(206,207,233));
+//  pal.setColor(QPalette::Button, QColor(206,207,233));
 #endif
   mLayout = new QGridLayout(mFrame);
-  int i;
+  int i, k;
   for (i = 0; i < _symbols.size(); ++i) {
-    QPixmap p = KLFLatexSymbolsCache::theCache()->getPixmap(_symbols[i]);
+    QPixmap p = cache->getSymbolPixmap(_symbols[i]);
     KLFPixmapButton *btn = new KLFPixmapButton(p, mFrame);
-#ifdef Q_WS_MAC
-    btn->setStyle(myStyle);
-    btn->setPalette(pal);
+#ifdef KLF_WS_MAC
+    btn->setStyle(buttonstyle);
+//    btn->setPalette(pal);
 #endif
     btn->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));
     btn->setProperty("symbol", QVariant::fromValue<int>(i));
     btn->setProperty("gridpos", QPoint(-1,-1));
     btn->setProperty("gridcolspan", -1);
     btn->setProperty("myWidth", p.width() + 4);
+    QString symcode = _symbols[i].symbol;
+    if (_symbols[i].symbol_option) {
+      symcode += "[]";
+    }
+    for (k = 0; _symbols[i].symbol_numargs > 0 && k < _symbols[i].symbol_numargs; ++k) {
+      symcode += "{}";
+    }
     QString tooltiptext =
       "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\""
       " \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
@@ -518,12 +599,17 @@ void KLFLatexSymbolsView::buildDisplay()
       "</style>"
       "</head>"
       "<body>\n"
-      "<p style=\"white-space: pre\">"+tr("LaTeX code:")+" <b><tt>"+_symbols[i].symbol+"</tt></b>"+
+      "<p style=\"white-space: pre\">"+tr("LaTeX code:")+" <b><tt>"+symcode+"</tt></b>"+
       (_symbols[i].textmode?tr(" [in text mode]"):QString(""))+
       +"</p>";
-    if (_symbols[i].preamble.size())
+    if (_symbols[i].preamble.size()) {
       tooltiptext += "<p>"+tr("Requires:")+"<b><pre>" +
 	_symbols[i].preamble.join("\n")+"</pre></b></p>";
+    }
+    if (_symbols[i].keywords.size()) {
+      tooltiptext += "<p>"+tr("Key Words:")+"<pre><b>" +
+	_symbols[i].keywords.join("</b>, <b>")+"</b></pre></p>";
+    }
     tooltiptext += "</body></html>";
     btn->setToolTip(tooltiptext);
     //klfDbg("tooltip text is "<<tooltiptext);
@@ -586,43 +672,36 @@ void KLFLatexSymbolsView::slotSymbolActivated()
 }
 
 
-bool KLFLatexSymbolsView::searchIterMatches(const SearchIterator& pos, const QString& queryString)
+static bool stringlist_contains(const QStringList& l, const QString& qs, Qt::CaseSensitivity cs)
 {
-  // remember:  SearchIterator==int
-  if (pos < 0 || pos >= mSymbols.size())
+  QRegExp qsrx(".*"+QRegExp::escape(qs)+".*", cs);
+  return l.indexOf(qsrx) >= 0;
+}
+
+bool KLFLatexSymbolsView::symbolMatches(int symbol, const QString& curqstr)
+{
+  if (symbol < 0 || symbol >= mSymbols.size())
     return false;
 
-  int symIndex = mSymbols[pos]->property("symbol").toInt();
+  int symIndex = mSymbols[symbol]->property("symbol").toInt();
   if (symIndex < 0 || symIndex >= _symbols.size()) {
     qWarning()<<KLF_FUNC_NAME<<": Inavlid symbol index "<<symIndex;
     return false;
   }
 
   // (X)Emacs-style: presence of capital letter triggers case sensitive search
-  Qt::CaseSensitivity cs = (queryString.contains(QRegExp("[A-Z]")) ? Qt::CaseSensitive : Qt::CaseInsensitive) ;
+  Qt::CaseSensitivity cs = (curqstr.contains(QRegExp("[A-Z]")) ? Qt::CaseSensitive : Qt::CaseInsensitive) ;
 
-  if ( _symbols[symIndex].symbol.contains(queryString, cs) ||
-       _symbols[symIndex].preamble.contains(queryString, cs) ) {
+  if ( _symbols[symIndex].symbol.contains(curqstr, cs) ||
+       stringlist_contains(_symbols[symIndex].preamble, curqstr, cs) ||
+       stringlist_contains(_symbols[symIndex].keywords, curqstr, cs) ) {
     klfDbg("found match at "<<symIndex<<": "<<_symbols[symIndex].symbol) ;
     return true;
   }
   return false;
 }
 
-void KLFLatexSymbolsView::searchPerformed(const SearchIterator& result)
-{
-  klfDbg("result is "<<result<<" valid="<<(result<mSymbols.size())) ;
-
-  highlightSearchMatches(result);
-}
-void KLFLatexSymbolsView::searchAbort()
-{
-  KLFIteratorSearchable<int>::searchAbort();
-  highlightSearchMatches(-1);
-  //  setFocus();
-}
-
-void KLFLatexSymbolsView::highlightSearchMatches(int currentMatch)
+void KLFLatexSymbolsView::highlightSearchMatches(int currentMatch, const QString& curqstr)
 {
   QString stylesheets[] = {
     // don't affect tooltips: give KLFPixmapButton { } scopes
@@ -640,21 +719,20 @@ void KLFLatexSymbolsView::highlightSearchMatches(int currentMatch)
     int which = 0;
     if (k == currentMatch)
       which = 2;
-    else if (searchIterMatches(k, searchQueryString()))
+    else if (curqstr.size() && symbolMatches(k, curqstr))
       which = 1;
     mSymbols[k]->setStyleSheet(stylesheets[which]);
   }  
-  if (currentMatch >= 0 && currentMatch < mSymbols.size())
-    ensureWidgetVisible(mSymbols[currentMatch]);
 }
 
 
 
 
 
+
 KLFLatexSymbols::KLFLatexSymbols(QWidget *parent, const KLFBackend::klfSettings& baseSettings)
   : QWidget(
-#if defined(Q_OS_WIN32)
+#if defined(KLF_WS_WIN) || defined(KLF_WS_MAC)
 	    0 /* parent */
 #else
 	    parent /* 0 */
@@ -662,64 +740,145 @@ KLFLatexSymbols::KLFLatexSymbols(QWidget *parent, const KLFBackend::klfSettings&
 	    , /*Qt::Tool*/ Qt::Window /*0*/)
 {
   KLF_DEBUG_TIME_BLOCK(KLF_FUNC_NAME) ;
+  Q_UNUSED(parent) ;
 
+  // set up private stuff
+  KLF_INIT_PRIVATE(KLFLatexSymbols) ;
+
+  // set up UI
   u = new Ui::KLFLatexSymbols;
   u->setupUi(this);
   setAttribute(Qt::WA_StyledBackground);
 
+  qreal device_pixel_ratio = devicePixelRatio();
+  klfDbg("device pixel ratio = " << device_pixel_ratio) ;
+
+  // create and set up the cache
+  d->pCache = new KLFLatexSymbolsCache(this);
+  d->pCache->setDevicePixelRatio( device_pixel_ratio );
+  d->pCache->loadKlfCache();
+
+  // set up the preview generator
+  d->pPreviewsGenerator = new KLFLatexSymbolPreviewsGenerator; // leave w/o parent as in basic QThread example
+  d->pPreviewsGenerator->setBackendSettings(baseSettings);
+  d->pPreviewsGenerator->setDevicePixelRatio( device_pixel_ratio );
+  d->pPreviewsGenerator->moveToThread(& d->workerThread);
+  connect(d->pPreviewsGenerator, SIGNAL(started()), u->wGenPreviews, SLOT(show()));
+  connect(d->pPreviewsGenerator, SIGNAL(started()), u->frmMain, SLOT(hide()));
+  connect(d->pPreviewsGenerator, SIGNAL(progress(int)), u->prgGenPreviews, SLOT(setValue(int)));
+  connect(d->pPreviewsGenerator, SIGNAL(previewGenerated(const KLFLatexSymbol&, const QPixmap &)),
+          d, SLOT(previewGenerated(const KLFLatexSymbol & , const QPixmap & )));
+  connect(u->btnGenPreviewsSkip, SIGNAL(clicked()), d, SLOT(skipGenPreviews()));
+  connect(d->pPreviewsGenerator, SIGNAL(finished()), d, SLOT(generatePreviewsFinished()));
+
   // add our search bar
-  pSearchBar = new KLFSearchBar(this);
-  KLF_DEBUG_ASSIGN_REF_INSTANCE(pSearchBar, "latexsymbols-searchbar") ;
-  pSearchBar->setShowOverlayMode(true);
-  pSearchBar->registerShortcuts(this);
-  pSearchBar->setSearchText("");
-  pSearchBar->setShowHideButton(true);
-  connect(pSearchBar, SIGNAL(escapePressed()), pSearchBar, SLOT(hide()));
+  KLF_DEBUG_ASSIGN_REF_INSTANCE(u->searchBar, "latexsymbols-searchbar") ;
+  //  u->searchBar->setShowOverlayMode(true);
+  u->searchBar->registerShortcuts(this);
+  u->searchBar->setShowHideButton(true);
+  connect(u->searchBar, SIGNAL(escapePressed()), u->searchBar, SLOT(hide()));
+
+  d->pSearchable = new KLFLatexSymbolsSearchable(this);
+  u->searchBar->setSearchTarget(d->pSearchable);
+
+  u->btnInsertSearchCurrent->hide();
 
   klfDbg("prepared search bar.") ;
 
-  KLFLatexSymbolsCache::theCache()->setBackendSettings(baseSettings);
-
-  // read our config and create the UI
-  read_symbols_create_ui();
-
-  slotShowCategory(0);
-
-  QFont f = u->cbxCategory->font();
-  int ps = f.pointSize();
-  if (ps < 8)
-    ps = QFontInfo(f).pointSize();
-  f.setPointSize(ps+1);
-  u->cbxCategory->setFont(f);
+  KLFRelativeFont *relfont = new KLFRelativeFont(u->cbxCategory);
+  relfont->setRelPointSize(+2);
 
   connect(u->cbxCategory, SIGNAL(highlighted(int)), this, SLOT(slotShowCategory(int)));
   connect(u->cbxCategory, SIGNAL(activated(int)), this, SLOT(slotShowCategory(int)));
   connect(u->btnClose, SIGNAL(clicked()), this, SLOT(close()));
+
+  connect(u->btnInsertSearchCurrent, SIGNAL(clicked()),
+	  d->pSearchable, SLOT(slotInsertCurrentMatch()));
+
+#ifdef KLF_WS_MAC
+  // Cmd+Space is used by spotlight, so use Ctrl+Space (on Qt/Mac, Ctrl key is mapped to Qt::META)
+  u->btnInsertSearchCurrent->setShortcut(QKeySequence(Qt::META + Qt::Key_Space));
+#endif
 }
 
 void KLFLatexSymbols::retranslateUi(bool alsoBaseUi)
 {
-  if (alsoBaseUi)
+  if (alsoBaseUi) {
     u->retranslateUi(this);
+  }
 }
 
 KLFLatexSymbols::~KLFLatexSymbols()
 {
-  KLFLatexSymbolsCache::saveTheCache();
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+
+  d->pCache->saveToKlfCache();
+
+  klfDbg("thread interrupt ...") ;
+  d->workerThread.requestInterruption();
+  klfDbg("thread quit ...") ;
+  d->workerThread.quit();// although it seems quit() needs to be called from within the thread... ??
+  klfDbg("thread wait ...") ;
+  d->workerThread.wait();
+
+  klfDbg("thread done.") ;
+
+  KLF_DELETE_PRIVATE ;
 }
 
-void KLFLatexSymbols::read_symbols_create_ui()
+void KLFLatexSymbols::load()
 {
-  klfDbgT("called.") ;
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+
+  // read our config and create the UI
+  d->read_symbols_create_ui();
+
+  slotShowCategory(0);
+
+  slotKlfConfigChanged();
+}
+
+void KLFLatexSymbols::emitInsertSymbol(const KLFLatexSymbol& symbol)
+{
+  emit insertSymbol(symbol);
+}
+
+void KLFLatexSymbols::slotKlfConfigChanged()
+{
+  u->searchBar->setEmacsStyleBackspace(klfconfig.UI.emacsStyleBackspaceSearch);
+}
+
+KLFLatexSymbolsCache * KLFLatexSymbols::cache()
+{
+  return d->pCache;
+}
+
+QList<KLFLatexSymbolsView *> KLFLatexSymbols::categoryViews()
+{
+  return d->mViews;
+}
+
+int KLFLatexSymbols::currentCategoryIndex() const
+{
+  return u->cbxCategory->currentIndex();
+}
+
+void KLFLatexSymbolsPrivate::read_symbols_create_ui()
+{
+  KLF_DEBUG_TIME_BLOCK(KLF_FUNC_NAME) ;
 
   // create our UI
-  u->cbxCategory->clear();
-  QGridLayout *lytstk = new QGridLayout(u->frmStackContainer);
-  stkViews = new QStackedWidget(u->frmStackContainer);
-  lytstk->addWidget(stkViews, 0, 0);
-
+  K->u->cbxCategory->clear();
+  // clear any existing views
+  foreach (KLFLatexSymbolsView * view, mViews) {
+    K->u->stkViews->removeWidget(view);
+  }
+  KLF_ASSERT_CONDITION(K->u->stkViews->count() == 0,
+                       "Widgets still left in view stack !! symbol categories will be wrong", ) ;
   mViews.clear();
 
+  // read symbols config
+
   // find collection of XML files
   QStringList fxmllist;
   // in the following directories
@@ -739,19 +898,6 @@ void KLFLatexSymbols::read_symbols_create_ui()
       fxmllist << fxmldir.absoluteFilePath(xmllist[j]);
   }
   klfDbgT("files collected: "<<fxmllist) ;
-  if (fxmllist.isEmpty()) {
-    // copy legacy XML file into the home latexsymbols.d directory
-    QDir("/").mkpath(klfconfig.homeConfigDir+"/conf/latexsymbols.d");
-    if (QFile::exists(klfconfig.homeConfigDir+"/latexsymbols.xml")) {
-      QFile::copy(klfconfig.homeConfigDir+"/latexsymbols.xml",
-		  klfconfig.homeConfigDir+"/conf/latexsymbols.d/mylatexsymbols.xml");
-      fxmllist << klfconfig.homeConfigDir+"/conf/latexsymbols.d/mylatexsymbols.xml";
-    } else {
-      QFile::copy(":/data/latexsymbols.xml",
-		  klfconfig.homeConfigDir+"/conf/latexsymbols.d/defaultlatexsymbols.xml");
-      fxmllist << klfconfig.homeConfigDir+"/conf/latexsymbols.d/defaultlatexsymbols.xml";
-    }
-  }
 
   klfDbgT("got xml files, ensured not empty; fxmllist="<<fxmllist) ;
 
@@ -783,11 +929,22 @@ void KLFLatexSymbols::read_symbols_create_ui()
     file.close();
     
     QDomElement root = doc.documentElement();
-    if (root.nodeName() != "latexsymbollist") {
-      qWarning("%s: Error parsing XML for latex symbols from file `%s': unexpected root tag `%s'.\n", KLF_FUNC_NAME,
+    if (root.nodeName() == "latexsymbollist") {
+      if (!fn.endsWith("conf/latexsymbols.d/defaultlatexsymbols.xml"))
+        klfWarning("Ignoring old symbols definition file "<<fn<<".");
+      continue;
+    }
+    if (root.nodeName() != "klf-symbol-list") {
+      qWarning("%s: Error parsing XML for latex symbols from file `%s': unexpected root tag `%s'.", KLF_FUNC_NAME,
 	       qPrintable(fn), qPrintable(root.nodeName()));
       continue;
     }
+    QString minklfversion = root.attribute("minklfversion", "3.3.0a");
+    if (klfVersionCompare(minklfversion, KLF_VERSION_STRING) > 0) {
+      qWarning("%s: ignoring XML latex symbols definition file `%s' which requires KLF version %s.",
+	       KLF_FUNC_NAME, qPrintable(fn), qPrintable(minklfversion));
+      continue;
+    }
 
     QDomNode n;
     for (n = root.firstChild(); ! n.isNull(); n = n.nextSibling()) {
@@ -818,16 +975,16 @@ void KLFLatexSymbols::read_symbols_create_ui()
 		 <<"; current category-title="<<categoryTitle<<",lang="<<curCategoryTitleLang) ;
 	  if (categoryTitle.isEmpty()) {
 	    // no category title yet
-	    if (lang.isEmpty() || lang.startsWith(klfconfig.UI.locale) || klfconfig.UI.locale.startsWith(lang)) {
+	    if (lang.isEmpty() || lang.startsWith(klfconfig.UI.locale) || klfconfig.UI.locale().startsWith(lang)) {
 	      // correct locale
 	      categoryTitle = qApp->translate("xmltr_latexsymbols", eesym.text().toUtf8().constData(),
-					      "[[tag: <category-title>]]", QCoreApplication::UnicodeUTF8);
+					      "[[tag: <category-title>]]");
 	      curCategoryTitleLang = lang;
 	    }
 	    // otherwise skip this tag
 	  } else {
 	    // see if this locale is correct and more specific
-	    if ( (lang.startsWith(klfconfig.UI.locale) || klfconfig.UI.locale.startsWith(lang)) &&
+	    if ( (lang.startsWith(klfconfig.UI.locale) || klfconfig.UI.locale().startsWith(lang)) &&
 		 (curCategoryTitleLang.isEmpty() || lang.startsWith(curCategoryTitleLang) ) ) {
 	      // then keep it and replace the other
 	      categoryTitle = eesym.text();
@@ -845,7 +1002,10 @@ void KLFLatexSymbols::read_symbols_create_ui()
 	// read symbol
 	KLFLatexSymbol sym(eesym);
 	l.append(sym);
-	allsymbols.append(sym);
+        if (!pCache->contains(sym)) {
+          klfDbg("Adding symbol " << sym << " to generate-pixmap-list") ;
+          allsymbols.append(sym);
+        }
       }
       // and add this category, or append to existing category
       KLFLatexSymbolsView * view = NULL;
@@ -857,21 +1017,24 @@ void KLFLatexSymbols::read_symbols_create_ui()
       }
       if (view == NULL) {
 	// category does not yet exist
-	view = new KLFLatexSymbolsView(heading, stkViews);
+        klfDbg("Adding view for category " << heading) ;
+	view = new KLFLatexSymbolsView(heading, NULL);
 	connect(view, SIGNAL(symbolActivated(const KLFLatexSymbol&)),
-		this, SIGNAL(insertSymbol(const KLFLatexSymbol&)));
+		K, SIGNAL(insertSymbol(const KLFLatexSymbol&))) ;
 	mViews.append(view);
-	stkViews->addWidget(view);
-	if (categoryTitle.isEmpty())
+	K->u->stkViews->addWidget(view);
+	if (categoryTitle.isEmpty()) {
 	  categoryTitle = heading;
-	u->cbxCategory->addItem(categoryTitle, heading);
+        }
+	K->u->cbxCategory->addItem(categoryTitle, heading);
 	categoryTitleLangs << curCategoryTitleLang;
       } else {
+        klfDbg("Updating view for category " << heading) ;
 	// possibly update the title if a better translation is available
 	if (!categoryTitle.isEmpty() &&
 	    (categoryTitleLangs[j].isEmpty() || curCategoryTitleLang.startsWith(categoryTitleLangs[j]))) {
 	  // update the title
-	  u->cbxCategory->setItemText(j, categoryTitle);
+	  K->u->cbxCategory->setItemText(j, categoryTitle);
 	} else {
 	  // keep old title
 	}
@@ -881,14 +1044,58 @@ void KLFLatexSymbols::read_symbols_create_ui()
     } // iterate over categories in XML file
   } // iterate over XML files
 
-  // pre-cache all our symbols
-  KLFLatexSymbolsCache::theCache()->precacheList(allsymbols, true, this);
 
-  int i;
-  for (i = 0; i < mViews.size(); ++i) {
-    mViews[i]->buildDisplay();
+  klfDbg("prepared symbols list, about to start worker thread") ;
+
+  workerThread.start();
+
+  klfDbg("worker thread started, about to call generatePreviewList (queued connection)") ;
+
+  QMetaObject::invokeMethod(pPreviewsGenerator, "generatePreviewList", Qt::QueuedConnection,
+                            Q_ARG(QList<KLFLatexSymbol>, allsymbols)) ;
+}
+
+void KLFLatexSymbolsPrivate::previewGenerated(const KLFLatexSymbol & symbol, const QPixmap & pixmap)
+{
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+  klfDbg("setting pixmap for symbol " << symbol.symbol) ;
+  pCache->setSymbolPixmap(symbol, pixmap);
+}
+
+void KLFLatexSymbolsPrivate::skipGenPreviews()
+{
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+  workerThread.requestInterruption();
+
+  // build whatever UI we can
+  foreach (KLFLatexSymbolsView * view, mViews) {
+    view->buildDisplay(pCache);
   }
 
+  K->u->wGenPreviews->hide();
+  K->u->frmMain->show();
+}
+
+void KLFLatexSymbolsPrivate::generatePreviewsFinished()
+{
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+
+  klfDbg("stopping thread.") ;
+  workerThread.quit();
+
+#ifdef KLF_DEBUG
+  // and dump symbol cache
+  pCache->debugDump();
+#endif
+
+  foreach (KLFLatexSymbolsView * view, mViews) {
+    klfDbg("setting up display for category " << view->category()) ;
+    view->buildDisplay(pCache);
+  }
+
+  klfDbg("hiding wGenPreviews widget") ;
+  K->u->wGenPreviews->hide();
+  K->u->frmMain->show();
 }
 
 void KLFLatexSymbols::slotShowCategory(int c)
@@ -896,18 +1103,9 @@ void KLFLatexSymbols::slotShowCategory(int c)
   KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
 
   // called by combobox
-  stkViews->setCurrentIndex(c);
+  u->stkViews->setCurrentIndex(c);
 
   klfDbg("current index="<<c) ;
-
-  QWidget * w = stkViews->currentWidget();
-  KLFSearchable * target = NULL;
-  if (w != NULL) {
-    KLFLatexSymbolsView *view = qobject_cast<KLFLatexSymbolsView*>(w);
-    if (view != NULL)
-      target = view;
-  }
-  pSearchBar->setSearchTarget(target);
 }
 
 void KLFLatexSymbols::closeEvent(QCloseEvent *e)
