@@ -19,7 +19,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-/* $Id: klflibbrowser_p.h 603 2011-02-26 23:14:55Z phfaist $ */
+/* $Id: klflibbrowser_p.h 952 2016-12-26 07:36:43Z phfaist $ */
 
 /** \file
  * This header contains (in principle _private_) definitions for klflibbrowser.cpp
@@ -37,7 +37,8 @@
 #include <QDragMoveEvent>
 #include <QDialog>
 #include <QStackedLayout>
-#include <QDir>
+#include <QUrl>
+#include <QUrlQuery>
 #include <QFileDialog>
 #include <QFileInfo>
 #include <QAction>
@@ -54,7 +55,7 @@
 #include "klflibbrowser.h"
 
 /** \internal */
-class KLFLibBrowserViewContainer : public QStackedWidget, public KLFSearchableProxy
+class KLFLibBrowserViewContainer : public QStackedWidget, public KLFPosSearchableProxy
 {
   Q_OBJECT
 public:
@@ -229,8 +230,10 @@ public:
       else
 	t = subreslist[k];
 
+      QUrlQuery urlq(baseurl);
+      urlq.addQueryItem("klfDefaultSubResource", subreslist[k]);
       QUrl url = baseurl;
-      url.addQueryItem("klfDefaultSubResource", subreslist[k]);
+      url.setQuery(urlq);
       QString urlstr = url.toString();
 
       QAction *a = NULL;
@@ -772,7 +775,9 @@ public:
 	for (j = 0; j < resItem->rowCount(); ++j) {
 	  if (resItem->child(j)->checkState() != Qt::Unchecked) {
 	    QUrl url = resItem->data(DataRole).toString();
-	    url.addQueryItem("klfDefaultSubResource", resItem->child(j)->data(DataRole).toString());
+            QUrlQuery urlq(url);
+	    urlq.addQueryItem("klfDefaultSubResource", resItem->child(j)->data(DataRole).toString());
+            url.setQuery(urlq);
 	    urlList << url;
 	    continue;
 	  }
@@ -861,8 +866,9 @@ private:
       KLFLibResourceEngine *res = pLibBrowser->getOpenResource(openurls[k]);
       QStandardItem *parent = getResourceParentItem(res);
       QStringList subreslist;
-      if (openurls[k].hasQueryItem("klfDefaultSubResource")) {
-	subreslist = QStringList() << openurls[k].queryItemValue("klfDefaultSubResource");
+      QUrlQuery urlq(openurls[k]);
+      if (urlq.hasQueryItem("klfDefaultSubResource")) {
+	subreslist = QStringList() << urlq.queryItemValue("klfDefaultSubResource");
       } else if (res->supportedFeatureFlags() & KLFLibResourceEngine::FeatureSubResources) {
 	// no default sub-resource is specified, so add them all
 	subreslist = res->subResourceList();
