@@ -19,10 +19,12 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-/* $Id: klfliblegacyengine.cpp 627 2011-04-12 12:36:22Z phfaist $ */
+/* $Id: klfliblegacyengine.cpp 953 2016-12-27 00:13:10Z phfaist $ */
 
 
 #include <QString>
+#include <QUrl>
+#include <QUrlQuery>
 #include <QObject>
 #include <QDataStream>
 #include <QDir>
@@ -144,33 +146,6 @@ KLF_EXPORT bool resources_equal_for_import(const KLFLegacyData::KLFLibraryResour
 }
 
 
-KLF_EXPORT QDataStream& operator<<(QDataStream& stream, const KLFLegacyData::KLFStyle& style)
-{
-  return stream << style.name << (quint32)style.fg_color << (quint32)style.bg_color
-		<< style.mathmode << style.preamble << (quint16)style.dpi;
-}
-KLF_EXPORT QDataStream& operator>>(QDataStream& stream, KLFLegacyData::KLFStyle& style)
-{
-  quint32 fg, bg;
-  quint16 dpi;
-  stream >> style.name;
-  stream >> fg >> bg >> style.mathmode >> style.preamble >> dpi;
-  style.fg_color = fg;
-  style.bg_color = bg;
-  style.dpi = dpi;
-  return stream;
-}
-KLF_EXPORT bool operator==(const KLFLegacyData::KLFStyle& a, const KLFLegacyData::KLFStyle& b)
-{
-  return a.name == b.name &&
-    a.fg_color == b.fg_color &&
-    a.bg_color == b.bg_color &&
-    a.mathmode == b.mathmode &&
-    a.preamble == b.preamble &&
-    a.dpi == b.dpi;
-}
-
-
 
 // -------------------------------------------
 
@@ -186,7 +161,9 @@ bool KLFLibLegacyFileDataPrivate::load(const QString& fnm)
 {
   KLF_DEBUG_TIME_BLOCK(KLF_FUNC_NAME+"("+fnm+")") ;
 
-  QString fname = (!fnm.isEmpty() ? fnm : filename);    
+  QString fname = (!fnm.isEmpty() ? fnm : filename);
+
+  flagForceReadOnly = false;
 
   klfDbg("loading from file "<<fname<<" (our filename="<<filename<<")") ;
 
@@ -247,6 +224,7 @@ bool KLFLibLegacyFileDataPrivate::load(const QString& fnm)
 	  
   } else {
     qWarning("Error: Library file `%s' is invalid library file or corrupt!", qPrintable(fname));
+    flagForceReadOnly = true;
     return false;
   }
 
@@ -266,7 +244,12 @@ bool KLFLibLegacyFileDataPrivate::load(const QString& fnm)
 
 bool KLFLibLegacyFileDataPrivate::save(const QString& fnm)
 {
-  KLF_DEBUG_TIME_BLOCK(KLF_FUNC_NAME+"("+fnm+")") ;
+  KLF_DEBUG_TIME_BLOCK(KLF_FUNC_NAME+"('"+fnm+"')") ;
+
+  if (flagForceReadOnly) {
+    klfWarning("attempt to save a forced-read-only file! fnm="<<fnm<<"/fileName="<<fileName()<<" !");
+    return false;
+  }
 
   QString fname = (!fnm.isEmpty() ? fnm : filename) ;
   klfDbg(" saving to file "<<fname<<" a "<<legacyLibType<<"-type library with N="<<resources.size()
@@ -342,8 +325,8 @@ bool KLFLibLegacyFileDataPrivate::save(const QString& fnm)
 
     stream << QString("KLATEXFORMULA_LIBRARY_EXPORT") << (qint16)2 << (qint16)1
 	   << resources << library;
-    // additionally, save our meta-data at the end (this will be ignored by previous versions
-    // of KLF)
+    // additionally, save our meta-data at the end (this will be ignored by versions of KLF which
+    // don't know about metadata...)
     stream << metadata;
 
     klfDbg("saved export-library type. resource count: "<<resources.size()) ;
@@ -415,8 +398,10 @@ KLFLibLegacyEngine * KLFLibLegacyEngine::openUrl(const QUrl& url, QObject *paren
   }
 
   QString legresname;
-  if (url.hasQueryItem("klfDefaultSubResource"))
-    legresname = url.queryItemValue("klfDefaultSubResource");
+  QUrlQuery urlq(url);
+  if (urlq.hasQueryItem("klfDefaultSubResource")) {
+    legresname = urlq.queryItemValue("klfDefaultSubResource");
+  }
 
   return new KLFLibLegacyEngine(fname, legresname, url, parent);
 }
@@ -429,12 +414,13 @@ KLFLibLegacyEngine * KLFLibLegacyEngine::createDotKLF(const QString& fname, QStr
 
   QString lrname = legacyResourceName;
   if (QFile::exists(fileName)) {
+    klfWarning("File "<<fileName<<" already exists!") ;
     // fail; we want to _CREATE_ a .klf file. Erase file before calling this function to overwrite.
     return NULL;
   }
 
   if (fileName.isEmpty()) {
-    qWarning()<<KLF_FUNC_NAME<<": file name "<<fileName<<" is empty!";
+    klfWarning("file name "<<fileName<<" is empty!");
     return NULL;
   }
   //  if (!QFileInfo(QFileInfo(fileName).absolutePath()).isWritable()) {
@@ -442,12 +428,27 @@ KLFLibLegacyEngine * KLFLibLegacyEngine::createDotKLF(const QString& fname, QStr
   //    return NULL;
   //  }
 
+  // create an empty .klf file
+  KLFLibLegacyFileDataPrivate * dd = KLFLibLegacyFileDataPrivate::instanceFor(fileName, false);
+  // initialize empty library
+  dd->legacyLibType = KLFLibLegacyFileDataPrivate::ExportLibraryType;
+  KLFLegacyData::KLFLibraryResource res = { KLFLegacyData::LibResourceUSERMIN, legacyResourceName };
+  dd->resources << res;
+  dd->library[res] = KLFLegacyData::KLFLibraryList();
+  // this will force a save
+  dd->flagForceReadOnly = false;
+  dd->haschanges = 1;
+  delete dd;
+  dd = NULL;
+
   QUrl url = QUrl::fromLocalFile(fileName);
   url.setScheme("klf+legacy");
 
   if (lrname.isEmpty())
     lrname = tr("Default Resource"); // default name...?
-  url.addQueryItem("klfDefaultSubResource", lrname);
+  QUrlQuery urlq(url);
+  urlq.addQueryItem("klfDefaultSubResource", lrname);
+  url.setQuery(urlq);
 
   klfDbgSt("fileName="<<fileName<<"; canonical file path="<<QFileInfo(fileName).canonicalFilePath()
 	   <<"; legacyResourceName="<<legacyResourceName);
@@ -486,6 +487,8 @@ KLFLibLegacyEngine::KLFLibLegacyEngine(const QString& fileName, const QString& r
     d->haschanges = true;
   }
 
+  setReadOnly(isReadOnly() || d->flagForceReadOnly);
+
   klfDbg("Opened KLFLibLegacyEngine resource `"<<fileName<<"': d="<<d<<"; resources="<<d->resources
 	 <<" (me="<<this<<", d="<<d<<")\n") ;
 }
@@ -524,7 +527,7 @@ bool KLFLibLegacyEngine::canModifyData(const QString& subResource, ModifyType mo
 
   KLF_ASSERT_NOT_NULL( d , "d is NULL!" , return false ) ;
 
-#ifndef Q_WS_WIN
+#ifndef KLF_WS_WIN
   // seems like windows doesn't like to test directories to be writable ...?
 
   if ( QFile::exists(d->fileName())  // depending on whether the file itself exists, check if
@@ -606,7 +609,7 @@ QStringList KLFLibLegacyEngine::subResourceList() const
 
 bool KLFLibLegacyEngine::canCreateSubResource() const
 {
-  // canModifyData is not sensitive to thses arguments...
+  // canModifyData is not sensitive to these arguments...
   return canModifyData(QString(), ChangeData);
 }
 bool KLFLibLegacyEngine::canRenameSubResource(const QString& subResource) const
@@ -835,7 +838,8 @@ bool KLFLibLegacyEngine::changeEntries(const QString& subResource, const QList<e
 	}
 	break;
       case KLFLibEntry::Style:
-	d->library[d->resources[index]][libindex].style = d->toLegacyStyle(values[j].value<KLFStyle>());
+	d->library[d->resources[index]][libindex].style =
+          KLFLegacyData::KLFLegacyStyle::fromNewStyle(values[j].value<KLFStyle>());
 	break;
       default:
 	qWarning()<<KLF_FUNC_NAME<<": Cannot set arbitrary property "<<propertyNameForId(properties[j])
@@ -947,14 +951,14 @@ void KLFLibLegacyEngine::updateResourceProperty(int propId)
     KLFPropertizedObject::setAllProperties(resprops);
     // set some default values for some properties if they have not been set
     if (!KLFPropertizedObject::property(PropLocked).isValid())
-      KLFPropertizedObject::setProperty(PropLocked, QVariant(false));
+      KLFPropertizedObject::doSetProperty(PropLocked, QVariant(false));
     if (!KLFPropertizedObject::property(PropAccessShared).isValid())
-      KLFPropertizedObject::setProperty(PropAccessShared, QVariant::fromValue(false));
+      KLFPropertizedObject::doSetProperty(PropAccessShared, QVariant::fromValue(false));
     if (!KLFPropertizedObject::property(PropTitle).isValid())
-      KLFPropertizedObject::setProperty(PropTitle, QFileInfo(d->fileName()).baseName());
+      KLFPropertizedObject::doSetProperty(PropTitle, QFileInfo(d->fileName()).baseName());
   } else {
     QString propName = KLFPropertizedObject::propertyNameForId(propId);
-    KLFPropertizedObject::setProperty(propId, resprops[propName]);
+    KLFPropertizedObject::doSetProperty(propId, resprops[propName]);
   }
   emit resourcePropertyChanged(propId);
 }
