@@ -19,7 +19,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-/* $Id: klflibview.h 603 2011-02-26 23:14:55Z phfaist $ */
+/* $Id: klflibview.h 604 2011-02-27 23:34:37Z phfaist $ */
 
 #ifndef KLFLIBVIEW_H
 #define KLFLIBVIEW_H
@@ -77,7 +77,7 @@ namespace KLFLib {
  *   the pointer before using it! See \ref validResourceEngine().
  *
  * If the reimplementation of this view can be searched with a KLFSearchBar, reimplement
- * searchable() to return a KLFSearchable object for this view.
+ * searchable() to return a KLFPosSearchable object for this view.
  */
 class KLF_EXPORT KLFAbstractLibView : public QWidget
 {
@@ -164,7 +164,7 @@ public:
    * categories. */
   virtual QStringList getCategorySuggestions() = 0;
 
-  virtual KLFSearchable * searchable() { return NULL; }
+  virtual KLFPosSearchable * searchable() { return NULL; }
 
 signals:
   /** Is emitted (by subclasses) when a latex entry is selected to be restored (eg. the entry was
@@ -638,6 +638,8 @@ private:
 
   bool dropCanInternal(const QMimeData *data);
 
+  friend class KLFLibViewSearchable;
+
   KLF_DEBUG_DECLARE_ASSIGNABLE_REF_INSTANCE() ;
 };
 
@@ -747,10 +749,12 @@ private:
 
 // -----------------
 
+class KLFLibViewSearchable;
+
 /** An implementation of the KLFAbstractLibView viwer to view library resource contents in
  * so-called Category, List or Icon view modes.
  */
-class KLF_EXPORT KLFLibDefaultView : public KLFAbstractLibView, public KLFSearchable
+class KLF_EXPORT KLFLibDefaultView : public KLFAbstractLibView
 {
   Q_OBJECT
   Q_PROPERTY(bool autoBackgroundItems READ autoBackgroundItems WRITE setAutoBackgroundItems) ;
@@ -797,7 +801,7 @@ public:
 
   virtual QStringList getCategorySuggestions();
 
-  virtual KLFSearchable * searchable() { return this; }
+  virtual KLFPosSearchable * searchable();
 
 public slots:
   //   virtual bool writeEntryProperty(int property, const QVariant& value);
@@ -805,10 +809,6 @@ public slots:
   //   virtual bool insertEntries(const KLFLibEntryList& entries);
   virtual bool selectEntries(const QList<KLFLib::entryId>& idList);
 
-  virtual bool searchFind(const QString& queryString, bool forward = true);
-  virtual bool searchFindNext(bool forward);
-  virtual void searchAbort();
-
   virtual void restore(uint restoreflags = KLFLib::RestoreLatexAndStyle);
 
   virtual void showColumns(int propIdColumn, bool show);
@@ -879,6 +879,8 @@ private:
   KLFLibViewDelegate *pDelegate;
   KLFLibModel *pModel;
 
+  KLFLibViewSearchable *pSearchable;
+
   bool pGroupSubCategories;
 
   QList<QAction*> pCommonActions;
@@ -894,8 +896,7 @@ private:
 
   bool func_selectAll(const QModelIndex& parent, uint flags, QTime *tm, KLFDelayedPleaseWaitPopup *pleaseWait);
 
-private slots:
-  void searchFound(const QModelIndex& i);
+  friend class KLFLibViewSearchable;
 
 protected:
   KLF_DEBUG_DECLARE_REF_INSTANCE( QFileInfo(url().path()).fileName()+":"
