@@ -19,7 +19,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-/* $Id: klflibview_p.h 754 2012-01-05 21:56:47Z phfaist $ */
+/* $Id: klflibview_p.h 983 2016-12-31 21:03:44Z phfaist $ */
 
 
 /** \file
@@ -50,12 +50,14 @@
 #include <QDirModel>
 #include <QCompleter>
 
-#include <ui_klfliblocalfilewidget.h>
+#include <klfiteratorsearchable.h>
 
 #include "klflib.h"
 #include "klflibview.h"
 #include "klfconfig.h"
 
+#include <ui_klfliblocalfilewidget.h>
+
 class KLFLibDefTreeView;
 
 
@@ -395,6 +397,110 @@ private:
 };
 
 
+// -----------------------------------------
+
+
+class KLFLibViewSearchable : public KLFIteratorSearchable<QModelIndex>
+{
+  KLFLibDefaultView *v;
+  inline KLFLibModel *m() { return v->pModel; }
+
+public:
+  KLFLibViewSearchable(KLFLibDefaultView *view)
+    : v(view)
+  {
+  }
+
+  SearchIterator searchIterBegin()
+  {
+    KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+    KLF_ASSERT_NOT_NULL(m(), "Model is NULL!", return QModelIndex() ) ;
+    return m()->walkNextIndex(QModelIndex());
+  }
+  SearchIterator searchIterEnd()
+  {
+    KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+    return QModelIndex();
+  }
+  SearchIterator searchIterAdvance(const SearchIterator& pos, bool forward)
+  {
+    KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+    KLF_ASSERT_NOT_NULL(m(), "Model is NULL!", return QModelIndex() ) ;
+    return forward ? m()->walkNextIndex(pos) : m()->walkPrevIndex(pos);
+  }
+
+  virtual SearchIterator searchIterStartFrom(bool /*forward*/)
+  {
+    KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+    KLF_ASSERT_NOT_NULL(m(), "Model is NULL!", return QModelIndex() ) ;
+
+    return v->currentVisibleIndex(true); // always first visible index
+  }
+
+  virtual bool searchIterMatches(const SearchIterator& pos, const QString& queryString)
+  {
+    KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+    KLF_ASSERT_NOT_NULL(m(), "Model is NULL!", return false ) ;
+    KLF_ASSERT_NOT_NULL(m()->pCache, "Model Cache is NULL!", return false ) ;
+
+    Qt::CaseSensitivity cs = Qt::CaseInsensitive;
+    if (queryString.contains(QRegExp("[A-Z]")))
+      cs = Qt::CaseSensitive;
+    KLFLibModelCache::NodeId n = m()->pCache->getNodeForIndex(pos);
+    return KLF_DEBUG_TEE( m()->pCache->searchNodeMatches(n, queryString, cs) ) ;
+  }
+
+  virtual void searchMoveToIterPos(const SearchIterator& pos)
+  {
+    KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+    v->pDelegate->setSearchIndex(pos);
+    if ( ! pos.isValid() ) {
+      //      v->pView->scrollToTop();
+      //      // unselect all
+      //      v->pView->selectionModel()->setCurrentIndex(QModelIndex(), QItemSelectionModel::Clear);
+      return;
+    } else {
+      v->pView->scrollTo(pos, QAbstractItemView::EnsureVisible);
+    }
+    if (v->pViewType == KLFLibDefaultView::CategoryTreeView) {
+      // if tree view, expand item
+      qobject_cast<QTreeView*>(v->pView)->expand(pos);
+    }
+    // select the item
+    v->pView->selectionModel()->setCurrentIndex(pos,
+						QItemSelectionModel::ClearAndSelect|
+						QItemSelectionModel::Rows);
+    v->updateDisplay();
+  }
+
+  virtual void searchPerformed(const SearchIterator& resultMatchPosition, bool found, const QString& queryString)
+  {
+    KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+    Q_UNUSED(resultMatchPosition) ;
+    Q_UNUSED(found) ;
+    v->pDelegate->setSearchString(queryString);
+  }
+  virtual void searchAborted()
+  {
+    KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+    v->pDelegate->setSearchString(QString());
+    v->pDelegate->setSearchIndex(QModelIndex());
+    v->updateDisplay(); // repaint widget to update search underline
+    KLFIteratorSearchable<QModelIndex>::searchAborted();
+  }
+
+  virtual void searchReinitialized()
+  {
+    KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+    v->pDelegate->setSearchString(QString());
+    v->pDelegate->setSearchIndex(QModelIndex());
+    v->updateDisplay();
+    KLFIteratorSearchable<QModelIndex>::searchReinitialized();
+  }
+
+
+  KLF_DEBUG_DECLARE_REF_INSTANCE("libview-searchable") ;
+};
 
 
 
@@ -443,6 +549,7 @@ public:
 
   /** \warning Caller eventFilter() must ensure not to recurse with fake events ! */  
   virtual bool evDragMove(QDragMoveEvent *de, const QPoint& pos) {
+    Q_UNUSED(pos) ;
     uint fl = pModel->dropFlags(de, thisView());
     klfDbg( "KLFLibDefViewCommon::evDragMove: flags are "<<fl<<"; pos is "<<pos ) ;
     // decide whether to accept the drop or to ignore it
@@ -474,6 +581,7 @@ public:
       //	qApp->sendEvent(object, event);
       // move the objects ourselves because of bug (?) in Qt's handling?
       QPoint delta = pos - mousePressedContentsPos;
+      Q_UNUSED( delta ) ;
       klfDbg( "Delta is "<<delta ) ;
       // and fake a QDragLeaveEvent
       QDragLeaveEvent fakeevent;
@@ -532,48 +640,46 @@ public:
 
 
   QModelIndex curVisibleIndex(bool firstOrLast) const {
-    Q_UNUSED(firstOrLast)
-
-    // In the future, firstOrLast will be used to search forwards or backwards from the right point
-    klfDbg( "curVisibleIndex: offset y is "<<scrollOffset().y() ) ;
-    QModelIndex it = QModelIndex();
-    while ((it = pModel->walkNextIndex(it)).isValid()) {
+    /*
+      int off_y = scrollOffset().y();
+      klfDbg( "curVisibleIndex: offset y is "<<off_y ) ;
+      QModelIndex it = QModelIndex();
+      while ((it = pModel->walkNextIndex(it)).isValid()) {
       klfDbg( "\texploring item it="<<it<<"; bottom="<<thisConstView()->visualRect(it).bottom() ) ;
       if (thisConstView()->visualRect(it).bottom() >= 0) {
-	// first index from the beginning, that is after our scroll offset.
+      // first index from the beginning, that is after our scroll offset.
       return it;
       }
-    }
-    return pModel->walkNextIndex(QModelIndex());
-    /*
-      QModelIndex index;
-      QPoint offset = scrollOffset();
-      klfDbg( " offset="<<offset ) ;
-      int xStart, yStart;
-      int xStep, yStep;
-      if (firstOrLast) {
+      }
+    */
+    QModelIndex index;
+    QPoint offset = scrollOffset();
+    Q_UNUSED( offset ) ;
+    klfDbg( " offset="<<offset ) ;
+    int xStart, yStart;
+    int xStep, yStep;
+    if (firstOrLast) {
       xStep = 40;
       yStep = 40;
       xStart = xStep/2;
       yStart = yStep/2;
-      } else {
+    } else {
       xStep = -40;
       yStep = -40;
       xStart = thisConstView()->width() - xStep/2;
       yStart = thisConstView()->height() - yStep/2;
-      }
-      int xpos, ypos;
-      for (xpos = xStart; xpos > 0 && xpos < thisConstView()->width(); xpos += xStep) {
+    }
+    int xpos, ypos;
+    for (xpos = xStart; xpos > 0 && xpos < thisConstView()->width(); xpos += xStep) {
       for (ypos = yStart; ypos > 0 && ypos < thisConstView()->height(); ypos += yStep) {
-      if ((index = thisConstView()->indexAt(QPoint(xpos,ypos))).isValid()) {
-      klfDbg( ": Found index = "<<index<<" at pos=("<<xpos<<","<<ypos<<"); "
-      <<" with offset "<<offset ) ;
-      return firstOrLast ? pModel->walkPrevIndex(index) : pModel->walkNextIndex(index);
+	if ((index = thisConstView()->indexAt(QPoint(xpos,ypos))).isValid()) {
+	  klfDbg( ": Found index = "<<index<<" at pos=("<<xpos<<","<<ypos<<"); "
+		  <<" with offset "<<offset ) ;
+	  return firstOrLast ? pModel->walkPrevIndex(index) : pModel->walkNextIndex(index);
+	}
       }
-      }
-      }
-      return firstOrLast ? pModel->walkNextIndex(QModelIndex()) : pModel->walkPrevIndex(QModelIndex());
-    */
+    }
+    return firstOrLast ? pModel->walkNextIndex(QModelIndex()) : pModel->walkPrevIndex(QModelIndex());
   }
 
   virtual void modelInitialized() { }
@@ -585,13 +691,13 @@ protected:
   QPoint mousePressedContentsPos;
 
   virtual QModelIndexList commonSelectedIndexes() const = 0;
-  virtual void commonInternalDrag(Qt::DropActions a) = 0;
+  //  virtual void commonInternalDrag(Qt::DropActions a) = 0;
   virtual QAbstractItemView *thisView() = 0;
   virtual const QAbstractItemView *thisConstView() const = 0;
   virtual QPoint scrollOffset() const = 0;
 
   /** Returns contents position */
-  virtual QPoint eventPos(QObject *object, QDragEnterEvent *event, int horoffset, int veroffset) {
+  QPoint eventPosBase(QObject *object, QDragEnterEvent *event, int horoffset, int veroffset) {
     if (object == thisView()->viewport())
       return event->pos() + QPoint(horoffset, veroffset);
     if (object == thisView())
@@ -689,13 +795,13 @@ public slots:
 
 protected:
   QModelIndexList commonSelectedIndexes() const { return selectedIndexes(); }
-  void commonInternalDrag(Qt::DropActions) {  }
+  // void commonInternalDrag(Qt::DropActions) {  }
   QAbstractItemView *thisView() { return this; }
   const QAbstractItemView *thisConstView() const { return this; }
   QPoint scrollOffset() const { return QPoint(horizontalOffset(), verticalOffset()); }
 
   QPoint eventPos(QObject *object, QDragEnterEvent *event) {
-    return KLFLibDefViewCommon::eventPos(object, event, horizontalOffset(), verticalOffset());
+    return eventPosBase(object, event, horizontalOffset(), verticalOffset());
   }
 
   void startDrag(Qt::DropActions supportedActions) {
@@ -772,13 +878,13 @@ public:
 
 protected:
   virtual QModelIndexList commonSelectedIndexes() const { return selectedIndexes(); }
-  virtual void commonInternalDrag(Qt::DropActions a) { internalDrag(a); }
+  //  virtual void commonInternalDrag(Qt::DropActions a) { internalDrag(a); }
   virtual QAbstractItemView *thisView() { return this; }
   virtual const QAbstractItemView *thisConstView() const { return this; }
   virtual QPoint scrollOffset() const { return QPoint(horizontalOffset(), verticalOffset()); }
 
-  virtual QPoint eventPos(QObject *object, QDragEnterEvent *event) {
-    return KLFLibDefViewCommon::eventPos(object, event, horizontalOffset(), verticalOffset());
+  QPoint eventPos(QObject *object, QDragEnterEvent *event) {
+    return eventPosBase(object, event, horizontalOffset(), verticalOffset());
   }
 
   bool inPaintEvent;
