@@ -19,19 +19,91 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-/* $Id: klfpobj.cpp 603 2011-02-26 23:14:55Z phfaist $ */
+/* $Id: klfpobj.cpp 991 2017-01-04 09:15:43Z phfaist $ */
 
-#include <QDebug>
+#include <QtDebug>
 #include <QByteArray>
 #include <QDataStream>
 #include <QTextStream>
-#include <QTextDocument>  // Qt::escape()
 
 #include "klfpobj.h"
 
+
+KLFAbstractPropertizedObject::KLFAbstractPropertizedObject()
+{
+}
+KLFAbstractPropertizedObject::~KLFAbstractPropertizedObject()
+{
+}
+
+KLFSpecifyableType::KLFSpecifyableType()
+{
+}
+KLFSpecifyableType::~KLFSpecifyableType()
+{
+}
+
+
+
+// -----------------------------
+
+
+QMap<QString,QVariant> KLFAbstractPropertizedObject::allProperties() const
+{
+  QMap<QString,QVariant> data;
+  QStringList pnames = propertyNameList();
+  foreach (QString pname, pnames) {
+    data[pname] = property(pname);
+  }
+  return data;
+}
+
+bool KLFAbstractPropertizedObject::setAllProperties(const QMap<QString,QVariant>& data)
+{
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+  klfDbg("data="<<data) ;
+  bool allok = true;
+  for (QVariantMap::const_iterator it = data.begin(); it != data.end(); ++it) {
+    bool ok = setProperty(it.key(), it.value());
+    if (!ok) {
+      allok = false;
+      qWarning()<<KLF_FUNC_NAME<<": Can't set property "<<it.key()<<" to "<<it.value();
+    }
+  }
+  return allok;
+}
+
+
+// --------
+
+KLF_EXPORT QDataStream& operator<<(QDataStream& stream, const KLFEnumType& e)
+{
+  return stream << e.specification() << qint32(e.value());
+}
+KLF_EXPORT QDataStream& operator>>(QDataStream& stream, KLFEnumType& e)
+{
+  QByteArray s;
+  qint32 x;
+  stream >> s >> x;
+  e.setSpecification(s);
+  e.setValue(x);
+  return stream;
+}
+
+
+// -------
+
+
+
+
 KLFPropertizedObject::KLFPropertizedObject(const QString& propNameSpace)
   : pPropNameSpace(propNameSpace)
 {
+  // ensure the property name space exists
+  if (!pRegisteredProperties.contains(propNameSpace))
+    pRegisteredProperties[propNameSpace] = QMap<QString,int>();
+  if (!pRegisteredPropertiesMaxId.contains(propNameSpace))
+    pRegisteredPropertiesMaxId[propNameSpace] = -1;
 }
 
 KLFPropertizedObject::~KLFPropertizedObject()
@@ -64,6 +136,31 @@ QVariant KLFPropertizedObject::property(int propId) const
   return QVariant();
 }
 
+QVariant KLFPropertizedObject::property(const QString& propname, const QVariant& defaultValue) const
+{
+  int propId = propertyIdForName(propname);
+  if (propId < 0) {
+    return defaultValue;
+  }
+  QVariant value = property(propId);
+  if (value.isValid())
+    return value;
+  return defaultValue;
+}
+
+bool KLFPropertizedObject::hasPropertyValue(const QString& propName) const
+{
+  return property(propName, QVariant()).isValid();
+}
+
+bool KLFPropertizedObject::hasPropertyValue(int propId) const
+{
+  if (!propertyIdRegistered(propId))
+    return false;
+
+  return hasPropertyValue(propertyNameForId(propId));
+}
+
 
 void KLFPropertizedObject::propertyValueChanged(int , const QVariant& ,
 						const QVariant& )
@@ -71,28 +168,28 @@ void KLFPropertizedObject::propertyValueChanged(int , const QVariant& ,
   // do nothing. Subclasses may implement thier own behavior.
 }
 
-void KLFPropertizedObject::setProperty(const QString& propname, const QVariant& value)
+bool KLFPropertizedObject::doSetProperty(const QString& propname, const QVariant& value)
 {
   if ( ! propertyNameRegistered(propname) ) {
     qWarning("%s[%s](): Property `%s' not registered.", KLF_FUNC_NAME, qPrintable(pPropNameSpace),
 	     qPrintable(propname));
-    return;
+    return false;
   }
-  setProperty(propertyIdForName(propname), value);
+  return doSetProperty(propertyIdForName(propname), value);
 }
-void KLFPropertizedObject::setProperty(int propId, const QVariant& value)
+bool KLFPropertizedObject::doSetProperty(int propId, const QVariant& value)
 {
   if (propId >= 0 && propId < pProperties.size()) {
     // all ok, set this property
     QVariant oldvalue = pProperties[propId];
     pProperties[propId] = value;
     propertyValueChanged(propId, oldvalue, value);
-    return;
+    return true;
   }
   if (propId < 0) {
     qWarning("%s[%s](id=%d): invalid property ID.", KLF_FUNC_NAME, qPrintable(pPropNameSpace),
 	     propId);
-    return;
+    return false;
   }
   // maybe our properties array needs resize for properties that could have been
   // registered after last access
@@ -104,14 +201,16 @@ void KLFPropertizedObject::setProperty(int propId, const QVariant& value)
       ! propertyIdRegistered(propId) ) {
     qWarning("%s[%s](id=%d): invalid property id.", KLF_FUNC_NAME, qPrintable(pPropNameSpace),
 	     propId);
-    return;
+    return false;
   }
   QVariant oldvalue = pProperties[propId];
   pProperties[propId] = value;
   propertyValueChanged(propId, oldvalue, value);
+  return true;
 }
-int KLFPropertizedObject::loadProperty(const QString& propname, const QVariant& value)
+int KLFPropertizedObject::doLoadProperty(const QString& propname, const QVariant& value)
 {
+  klfDbg("propname="<<propname<<" value="<<value) ;
   int propId = propertyIdForName(propname);
   if ( propId < 0 ) {
     // register property
@@ -119,7 +218,7 @@ int KLFPropertizedObject::loadProperty(const QString& propname, const QVariant&
     if (propId < 0)
       return -1;
   }
-  setProperty(propId, value);
+  doSetProperty(propId, value);
   return propId;
 }
 
@@ -162,13 +261,37 @@ QMap<QString,QVariant> KLFPropertizedObject::allProperties() const
   return properties;
 }
 
-void KLFPropertizedObject::setAllProperties(const QMap<QString, QVariant>& propValues)
+bool KLFPropertizedObject::setProperty(const QString& propname, const QVariant& value)
 {
+  return doLoadProperty(propname, value) >= 0;
+}
+
+bool KLFPropertizedObject::setProperty(int propId, const QVariant& value)
+{
+  KLF_ASSERT_CONDITION(propertyIdRegistered(propId), "Property ID="<<propId<<" is not registered!",
+		       return false; ) ;
+
+  return setProperty(propertyNameForId(propId), value);
+}
+
+
+bool KLFPropertizedObject::setAllProperties(const QMap<QString, QVariant>& propValues)
+{
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+  klfDbg("propValues="<<propValues) ;
+
+  bool allok = true;
   QStringList propKeys = propValues.keys();
   int k;
   for (k = 0; k < propKeys.size(); ++k) {
-    loadProperty(propKeys[k], propValues[propKeys[k]]);
+    // bypass check, set property anyway
+    bool ok = (doLoadProperty(propKeys[k], propValues[propKeys[k]]) >= 0);
+    if (!ok) {
+      allok = false;
+      qWarning()<<KLF_FUNC_NAME<<": Failed to load property "<<propKeys[k]<<" with value "<<propValues[propKeys[k]];
+    }
   }
+  return allok;
 }
 
 
@@ -189,6 +312,23 @@ void KLFPropertizedObject::setAllPropertiesFromByteArray(const QByteArray& data)
   stream >> *this;
 }
 
+/*
+QVariant KLFPropertizedObject::parsePropertyValue(int propId, const QString& strvalue)
+{
+  KLF_ASSERT_CONDITION(propertyIdRegistered(propId), "Property ID="<<propId<<" is not registered!",
+		       return QVariant(); ) ;
+
+  return parsePropertyValue(propertyNameForId(propId), strvalue);
+}
+
+QVariant KLFPropertizedObject::parsePropertyValue(const QString& / *propName* /, const QString& / *strvalue* /)
+{
+  return QVariant();
+}
+*/
+
+
+
 
 
 QString KLFPropertizedObject::toString(uint toStringFlags) const
@@ -210,11 +350,11 @@ QString KLFPropertizedObject::toString(uint toStringFlags) const
   if (html) {
     if (usehtmldiv) {
       s = QString("<div class=\"klfpobj_entry\">\n<div class=\"klfpobj_name\">%2</div>\n")
-	.arg(Qt::escape(pPropNameSpace));
+	.arg(pPropNameSpace.toHtmlEscaped());
     } else {
       s = QString("<table class=\"klfpobj_tentry\">\n"
 		  "<tr colspan=\"2\" class=\"klfpobj_tname\"><th>%1</th></tr>\n")
-	.arg(Qt::escape(pPropNameSpace));
+	.arg(pPropNameSpace.toHtmlEscaped());
     }
   } else {
     s = QString("%1\n").arg(pPropNameSpace);
@@ -230,11 +370,11 @@ QString KLFPropertizedObject::toString(uint toStringFlags) const
       if (usehtmldiv)
 	s += QString("<div class=\"klfpobj_prop_%1\"><div class=\"klfpobj_propname\">%2</div>: "
 		     "<div class=\"klfpobj_propvalue\">%3</div></div>\n")
-	  .arg(pname, pname, Qt::escape(value));
+	  .arg(pname, pname, value.toHtmlEscaped());
       else
 	s += QString("  <tr class=\"klfpobj_tprop_%1\"><td class=\"klfpobj_tpropname\">%2</td>"
 		     "<td class=\"klfpobj_tpropvalue\">%3</td></tr>\n")
-	  .arg(pname, pname, Qt::escape(value));
+	  .arg(pname, pname, value.toHtmlEscaped());
     } else {
       if (quote) {
 	if (!isnull && canstring) {
@@ -411,6 +551,9 @@ int KLFPropertizedObject::internalRegisterProperty(const QString& propNameSpace,
 						   const QString& propName,
 						   int propId)
 {
+  KLF_DEBUG_BLOCK(KLF_FUNC_NAME) ;
+  klfDbg("propNameSpace = " << propNameSpace << ", propName = " << propName << ", propId = " << propId) ;
+  
   const QMap<QString, int> propList = pRegisteredProperties[propNameSpace];
   int propMaxId = -1;
   if (pRegisteredPropertiesMaxId.contains(propNameSpace)) {
@@ -423,13 +566,15 @@ int KLFPropertizedObject::internalRegisterProperty(const QString& propNameSpace,
     propMaxId = propId;
   } else {
     // used the fixed propId. Update propMaxId if necessary.
-    if (propId > propMaxId)
+    if (propId > propMaxId) {
       propMaxId = propId;
+    }
   }
   if ( propList.keys(propId).size() > 0 ) {
     QString oldPropName = propList.keys(propId).at(0);
-    if (propName == oldPropName)
+    if (propName == oldPropName) {
       return propId; // already registered, return that property ID
+    }
     qWarning("%s[%s]: Property ID `%d' is already registered with conflicting names!\n"
 	     "\told name is `%s', new is `%s'",
 	     KLF_FUNC_NAME, qPrintable(propNameSpace), propId, qPrintable(oldPropName),
@@ -471,18 +616,26 @@ bool operator==(const KLFPropertizedObject& a, const KLFPropertizedObject& b)
 
 
 
-QDataStream& operator<<(QDataStream& stream, const KLFPropertizedObject& obj)
+QDataStream& KLFPropertizedObject::streamInto(QDataStream& stream) const
 {
-  stream << obj.allProperties();
+  stream << allProperties();
   return stream;
 }
-QDataStream& operator>>(QDataStream& stream, KLFPropertizedObject& obj)
+QDataStream& KLFPropertizedObject::streamFrom(QDataStream& stream)
 {
   QMap<QString,QVariant> props;
   stream >> props;
-  obj.setAllProperties(props);
+  setAllProperties(props);
   return stream;
 }
+QDataStream& operator<<(QDataStream& stream, const KLFPropertizedObject& obj)
+{
+  return obj.streamInto(stream);
+}
+QDataStream& operator>>(QDataStream& stream, KLFPropertizedObject& obj)
+{
+  return obj.streamFrom(stream);
+}
 
 
 QTextStream& operator<<(QTextStream& stream, const KLFPropertizedObject& obj)
@@ -498,3 +651,5 @@ QDebug& operator<<(QDebug& stream, const KLFPropertizedObject& obj)
   return stream;
 }
 
+
+
